@@ -6,8 +6,6 @@ import (
 	"reflect"
 	"time"
 
-	"github.com/ghodss/yaml"
-
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
 
@@ -20,6 +18,8 @@ import (
 	gcpdns "github.com/openshift/cluster-ingress-operator/pkg/dns/gcp"
 	logf "github.com/openshift/cluster-ingress-operator/pkg/log"
 	"github.com/openshift/cluster-ingress-operator/pkg/manifests"
+	operatorutil "github.com/openshift/cluster-ingress-operator/pkg/util"
+	oputil "github.com/openshift/cluster-ingress-operator/pkg/util"
 	"github.com/openshift/cluster-ingress-operator/pkg/util/slice"
 
 	corev1 "k8s.io/api/core/v1"
@@ -206,7 +206,7 @@ func (r *reconciler) createDNSProviderIfNeeded(dnsConfig *configv1.DNS) error {
 		return fmt.Errorf("failed to get infrastructure 'config': %v", err)
 	}
 
-	platformStatus, err := getPlatformStatus(r.client, infraConfig)
+	platformStatus, err := oputil.GetPlatformStatus(r.client, infraConfig)
 	if err != nil {
 		return fmt.Errorf("failed to get platform status: %v", err)
 	}
@@ -437,18 +437,65 @@ func (r *reconciler) createDNSProvider(dnsConfig *configv1.DNS, platformStatus *
 
 	switch platformStatus.Type {
 	case configv1.AWSPlatformType:
-		provider, err := awsdns.NewProvider(awsdns.Config{
+		cfg := awsdns.Config{
 			AccessID:  string(creds.Data["aws_access_key_id"]),
 			AccessKey: string(creds.Data["aws_secret_access_key"]),
 			Region:    platformStatus.AWS.Region,
-		}, r.Config.OperatorReleaseVersion)
+		}
+		if len(platformStatus.AWS.ServiceEndpoints) > 0 {
+			cfg.ServiceEndpoints = []awsdns.ServiceEndpoint{}
+			route53Found := false
+			elbFound := false
+			tagFound := false
+			for _, ep := range platformStatus.AWS.ServiceEndpoints {
+				switch {
+				case route53Found && elbFound && tagFound:
+					break
+				case ep.Name == awsdns.Route53Service:
+					route53Found = true
+					scheme, err := operatorutil.URI(ep.URL)
+					if err != nil {
+						return nil, fmt.Errorf("failed to validate URI %s: %v", ep.URL, err)
+					}
+					if scheme != operatorutil.SchemeHTTPS {
+						return nil, fmt.Errorf("invalid scheme for URI %s; must be %s", ep.URL, operatorutil.SchemeHTTPS)
+					}
+					cfg.ServiceEndpoints = append(cfg.ServiceEndpoints, awsdns.ServiceEndpoint{Name: ep.Name, URL: ep.URL})
+				case ep.Name == awsdns.ELBService:
+					elbFound = true
+					scheme, err := operatorutil.URI(ep.URL)
+					if err != nil {
+						return nil, fmt.Errorf("failed to validate URI %s: %v", ep.URL, err)
+					}
+					if scheme != operatorutil.SchemeHTTPS {
+						return nil, fmt.Errorf("invalid scheme for URI %s; must be %s", ep.URL, operatorutil.SchemeHTTPS)
+					}
+					cfg.ServiceEndpoints = append(cfg.ServiceEndpoints, awsdns.ServiceEndpoint{Name: ep.Name, URL: ep.URL})
+				case ep.Name == awsdns.TaggingService:
+					tagFound = true
+					scheme, err := operatorutil.URI(ep.URL)
+					if err != nil {
+						return nil, fmt.Errorf("failed to validate URI %s: %v", ep.URL, err)
+					}
+					if scheme != operatorutil.SchemeHTTPS {
+						return nil, fmt.Errorf("invalid scheme for URI %s; must be %s", ep.URL, operatorutil.SchemeHTTPS)
+					}
+					cfg.ServiceEndpoints = append(cfg.ServiceEndpoints, awsdns.ServiceEndpoint{Name: ep.Name, URL: ep.URL})
+				}
+			}
+		}
+		provider, err := awsdns.NewProvider(cfg, r.OperatorReleaseVersion)
 		if err != nil {
 			return nil, fmt.Errorf("failed to create AWS DNS manager: %v", err)
 		}
 		dnsProvider = provider
 	case configv1.AzurePlatformType:
+		environment := platformStatus.Azure.CloudName
+		if environment == "" {
+			environment = configv1.AzurePublicCloud
+		}
 		provider, err := azuredns.NewProvider(azuredns.Config{
-			Environment:    "AzurePublicCloud",
+			Environment:    string(environment),
 			ClientID:       string(creds.Data["azure_client_id"]),
 			ClientSecret:   string(creds.Data["azure_client_secret"]),
 			TenantID:       string(creds.Data["azure_tenant_id"]),
@@ -473,48 +520,3 @@ func (r *reconciler) createDNSProvider(dnsConfig *configv1.DNS, platformStatus *
 	}
 	return dnsProvider, nil
 }
-
-// getPlatformStatus provides a backwards-compatible way to look up platform status. AWS is the
-// special case. 4.1 clusters on AWS expose the region config only through install-config. New AWS clusters
-// and all other 4.2+ platforms are configured via platform status.
-func getPlatformStatus(client client.Client, infra *configv1.Infrastructure) (*configv1.PlatformStatus, error) {
-	if status := infra.Status.PlatformStatus; status != nil {
-		// Only AWS needs backwards compatibility with install-config
-		if status.Type != configv1.AWSPlatformType {
-			return status, nil
-		}
-
-		// Check whether the cluster config is already migrated
-		if status.AWS != nil && len(status.AWS.Region) > 0 {
-			return status, nil
-		}
-	}
-
-	// Otherwise build a platform status from the deprecated install-config
-	type installConfig struct {
-		Platform struct {
-			AWS struct {
-				Region string `json:"region"`
-			} `json:"aws"`
-		} `json:"platform"`
-	}
-	clusterConfigName := types.NamespacedName{Namespace: "kube-system", Name: "cluster-config-v1"}
-	clusterConfig := &corev1.ConfigMap{}
-	if err := client.Get(context.TODO(), clusterConfigName, clusterConfig); err != nil {
-		return nil, fmt.Errorf("failed to get configmap %s: %v", clusterConfigName, err)
-	}
-	data, ok := clusterConfig.Data["install-config"]
-	if !ok {
-		return nil, fmt.Errorf("missing install-config in configmap")
-	}
-	var ic installConfig
-	if err := yaml.Unmarshal([]byte(data), &ic); err != nil {
-		return nil, fmt.Errorf("invalid install-config: %v\njson:\n%s", err, data)
-	}
-	return &configv1.PlatformStatus{
-		Type: infra.Status.Platform,
-		AWS: &configv1.AWSPlatformStatus{
-			Region: ic.Platform.AWS.Region,
-		},
-	}, nil
-}
